@@ -41,6 +41,13 @@
 #include "utils.h"
 #include "glib-utils.h"
 
+typedef struct _KzBookmarkBarPrivate	KzBookmarkBarPrivate;
+struct _KzBookmarkBarPrivate
+{
+	GtkWidget *dragged_item;
+};
+#define KZ_BOOKMARK_BAR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_BOOKMARK_BAR, KzBookmarkBarPrivate))
+
 enum {
 	PROP_0,
 	PROP_KZ_WINDOW,
@@ -60,7 +67,7 @@ static GtkTargetEntry url_drag_types [] =
 	{ "text/uri-list",        0, TARGET_TEXT_URI_LIST}
 };
 
-static guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
+static const guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
 
 static void     kz_bookmark_bar_class_init   (KzBookmarkBarClass *klass);
 static void     kz_bookmark_bar_init         (KzBookmarkBar *bar);
@@ -86,12 +93,14 @@ static void     kz_bookmark_bar_drag_data_received
 					      GtkSelectionData *data,
 					      guint info,
 					      guint time);
-#if 0
+static void     kz_bookmark_bar_drag_leave   (GtkWidget *widget,
+					      GdkDragContext *context,
+					      guint time);
 static gboolean kz_bookmark_bar_drag_motion  (GtkWidget *widget,
 					      GdkDragContext *context,
 					      gint x, gint y,
 					      guint time);
-#endif
+
 static void     kz_bookmark_bar_refresh_all  (KzBookmarkBar *bar);
 
 static void     kz_bookmark_bar_connect_signal    (KzBookmarkBar *bar);
@@ -138,7 +147,8 @@ kz_bookmark_bar_class_init (KzBookmarkBarClass *klass)
 
 	widget_class->button_release_event = kz_bookmark_bar_button_release;
 	widget_class->drag_data_received   = kz_bookmark_bar_drag_data_received;
-/*	widget_class->drag_motion          = kz_bookmark_bar_drag_motion; */
+	widget_class->drag_leave           = kz_bookmark_bar_drag_leave;
+	widget_class->drag_motion          = kz_bookmark_bar_drag_motion;
 
 	g_object_class_install_property
 		(gobject_class,
@@ -159,15 +169,20 @@ kz_bookmark_bar_class_init (KzBookmarkBarClass *klass)
 				      KZ_TYPE_BOOKMARK,
 				      G_PARAM_READWRITE |
 				      G_PARAM_CONSTRUCT_ONLY));
+	g_type_class_add_private(gobject_class, sizeof(KzBookmarkBarPrivate));
 }
 
 
 static void
 kz_bookmark_bar_init (KzBookmarkBar *bar)
 {
+	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (bar);
 	bar->toolbar = gtk_toolbar_new();
 	bar->kz      = NULL;
 	bar->folder  = NULL;
+	
+	priv->dragged_item = GTK_WIDGET(gtk_separator_tool_item_new());
+	g_object_ref(priv->dragged_item);
 
 	gtk_container_add(GTK_CONTAINER(bar), bar->toolbar);
 	gtk_widget_show(bar->toolbar);
@@ -183,9 +198,7 @@ static void
 kz_bookmark_bar_dispose (GObject *object)
 {
 	KzBookmarkBar *bar = KZ_BOOKMARK_BAR(object);
-
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
+	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (bar);
 
 	if (bar->folder)
 	{
@@ -199,6 +212,16 @@ kz_bookmark_bar_dispose (GObject *object)
 		g_object_unref(bar->kz);
 		bar->kz = NULL;
 	}
+
+	if (priv->dragged_item)
+	{
+		gtk_widget_destroy(GTK_WIDGET(priv->dragged_item));
+		g_object_unref(priv->dragged_item);
+		priv->dragged_item = NULL;
+	}
+
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		G_OBJECT_CLASS (parent_class)->dispose(object);
 }
 
 
@@ -286,6 +309,7 @@ kz_bookmark_bar_button_release (GtkWidget *widget, GdkEventButton *event)
 						     bar->folder,
 						     event->button,
 						     event->time);
+		return TRUE;
 		break;
 	default:
 		break;
@@ -425,7 +449,7 @@ kz_bookmark_bar_drag_data_received (GtkWidget *widget,
 			kz_bookmark_append(bar->folder, bookmark);
 		}
 		kz_bookmark_file_save(KZ_BOOKMARK_FILE(bar->folder));
-		gtk_drag_finish(context, TRUE, FALSE, time);
+		gtk_drag_finish(context, TRUE, TRUE, time);
 	 	break;
 	 case TARGET_NETSCAPE_URL:
 	 case TARGET_TEXT_URI_LIST:
@@ -484,7 +508,19 @@ kz_bookmark_bar_drag_data_received (GtkWidget *widget,
 	}
 }
 
-#if 0
+static void
+kz_bookmark_bar_drag_leave   (GtkWidget *widget,
+			      GdkDragContext *context,
+			      guint time)
+{
+	KzBookmarkBar *bar;
+
+	bar = KZ_BOOKMARK_BAR(widget);
+	gtk_toolbar_set_drop_highlight_item(GTK_TOOLBAR(bar->toolbar),
+					    NULL, 0);
+}
+
+
 static gboolean 
 kz_bookmark_bar_drag_motion (GtkWidget *widget,
 			     GdkDragContext *context,
@@ -492,26 +528,20 @@ kz_bookmark_bar_drag_motion (GtkWidget *widget,
 			     guint time)
 {
 	KzBookmarkBar *bar;
+	KzBookmarkBarPrivate *priv = KZ_BOOKMARK_BAR_GET_PRIVATE (widget);
 	gint index;
-	
+
 	bar = KZ_BOOKMARK_BAR(widget);
 	index = gtk_toolbar_get_drop_index(GTK_TOOLBAR(bar->toolbar),
 					   x, y);
-	
+
 	gtk_toolbar_set_drop_highlight_item(GTK_TOOLBAR(bar->toolbar),
-					    NULL, index);
+					    GTK_TOOL_ITEM(priv->dragged_item), index);
 	
 	gdk_drag_status(context, context->suggested_action, time);
 
 	return TRUE;
 }
-#endif
-
-static void
-cb_toolitem_destroy(GtkWidget *widget, GtkWidget *proxy)
-{
-	gtk_widget_destroy(proxy);
-}
 
 
 static GtkToolItem *
@@ -523,30 +553,10 @@ create_tool_item (KzBookmarkBar *bar, KzBookmark *child)
 	g_return_val_if_fail(KZ_IS_BOOKMARK_BAR(bar), NULL);
 
 	if (kz_bookmark_is_separator(child))
-	{
 		toolitem = gtk_separator_tool_item_new();
-	}
 	else
-	{
-		GtkWidget *widget, *proxy;
-
-		widget = kz_bookmark_item_new(bar->kz, child);
-		gtk_widget_show(widget);
-		toolitem = gtk_tool_item_new();
-		gtk_container_add(GTK_CONTAINER(toolitem),
-				  GTK_WIDGET(widget));
-		proxy = kz_bookmark_menu_create_menuitem(child, bar->kz, FALSE);
-		gtk_tool_item_set_proxy_menu_item(GTK_TOOL_ITEM(toolitem),
-						  "bookmark", proxy);
-		/*
-		 * We need to connect "destroy" signal to disconnect signals 
-		 * to bookmark folder associated with proxy.
-		 */
-		g_signal_connect(toolitem, "destroy",
-				 G_CALLBACK(cb_toolitem_destroy), proxy);
-		
-		gtk_widget_show(proxy);
-	}
+		toolitem = kz_bookmark_item_new(bar->kz, child);
+	
 	gtk_widget_show(GTK_WIDGET(toolitem));
 	return toolitem;	
 }
