@@ -34,7 +34,6 @@ static void kz_file_get_property    (GObject *object,
 				     GValue *value,
 				     GParamSpec *pspec);
 static void kz_file_dispose         (GObject *object);
-static void kz_file_finalize        (GObject *object);
 
 static void      file_start      (KzIO *io);
 static GIOStatus read_from_io    (KzIO *io,
@@ -42,10 +41,12 @@ static GIOStatus read_from_io    (KzIO *io,
 static GIOStatus write_to_io     (KzIO *io,
 				  GIOChannel *iochannel);
 
+typedef struct _KzFilePrivate KzFilePrivate;
 struct _KzFilePrivate 
 {
 	gchar *filename;
 };
+#define KZ_FILE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_FILE, KzFilePrivate))
 
 enum {
 	PROP_0,
@@ -71,7 +72,6 @@ kz_file_class_init (KzFileClass *klass)
 	io_class     = (KzIOClass *) klass;
 
 	object_class->dispose      = kz_file_dispose;
-	object_class->finalize     = kz_file_finalize;
 	object_class->set_property = kz_file_set_property;
 	object_class->get_property = kz_file_get_property;
 	
@@ -88,52 +88,48 @@ kz_file_class_init (KzFileClass *klass)
 			_("The Filename of the local file"),
 			NULL,
 			G_PARAM_READWRITE));
+	g_type_class_add_private (object_class, sizeof(KzFilePrivate));
 }
 
 
 static void
 kz_file_init (KzFile *file)
 {
-	file->priv = g_new0(KzFilePrivate, 1);
-	
-	file->priv->filename = NULL;
+	KzFilePrivate *priv = KZ_FILE_GET_PRIVATE (file);
+
+	priv->filename = NULL;
 }
 
 
 static void
 kz_file_dispose (GObject *object)
 {
-	KzFile *file;
+	KzFilePrivate *priv = KZ_FILE_GET_PRIVATE (object);
 
-	file = KZ_FILE(object);
-
-	if (file->priv->filename)
+	if (priv->filename)
 	{
-		g_free(file->priv->filename);
+		g_free(priv->filename);
 	}
-	file->priv->filename = NULL;
+	priv->filename = NULL;
 
 	if (G_OBJECT_CLASS (parent_class)->dispose)
 		G_OBJECT_CLASS (parent_class)->dispose(object);
 }
 
 
-KZ_OBJECT_FINALIZE(kz_file, KzFile)
-
-
 static void
 kz_file_set_property (GObject *object,
 		      guint prop_id,
 		      const GValue *value,
 		      GParamSpec *pspec)
 {
-	KzFile *file = KZ_FILE(object);
+	KzFilePrivate *priv = KZ_FILE_GET_PRIVATE (object);
 
 	switch (prop_id)
 	{
 	case PROP_FILENAME:
-		g_free(file->priv->filename);
-		file->priv->filename = g_value_dup_string(value);
+		g_free(priv->filename);
+		priv->filename = g_value_dup_string(value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -148,12 +144,12 @@ kz_file_get_property (GObject *object,
 		      GValue *value,
 		      GParamSpec *pspec)
 {
-	KzFile *file = KZ_FILE(object);
+	KzFilePrivate *priv = KZ_FILE_GET_PRIVATE (object);
 
 	switch (prop_id)
 	{
 	case PROP_FILENAME:
-		g_value_set_string(value, file->priv->filename);
+		g_value_set_string(value, priv->filename);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -180,12 +176,10 @@ static GIOStatus
 kz_file_prepare_iochannel (KzIO *io)
 {
 	GIOStatus iostatus;
-	KzFile *file;
 	gchar *filename;
+	KzFilePrivate *priv = KZ_FILE_GET_PRIVATE (io);
 
-	file = KZ_FILE(io);
-	
-	filename = file->priv->filename;
+	filename = priv->filename;
 
 	if (kz_io_get_mode(io) == KZ_IO_READ)
 	{
