@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmarks-sidebar.c,v 1.11 2005/09/13 02:22:31 kous Exp $
+ *  $Id: kz-bookmarks-sidebar.c,v 1.12 2006/08/25 15:06:38 kous Exp $
  */
 
 #include "kz-bookmarks-sidebar.h"
@@ -48,6 +48,9 @@ static void     kz_bookmarks_sidebar_get_property (GObject      *object,
 						   GValue       *value,
 						   GParamSpec   *pspec);
 
+static GtkWidget *kz_bookmarks_sidebar_new_with_entry (KzSidebarEntry *entry,
+						       KzSidebar   *sidebar);
+
 static gboolean cb_button_release                 (GtkWidget          *widget,
 						   GdkEventButton     *event,
 						   KzBookmarksSidebar *sidebar);
@@ -58,7 +61,7 @@ static KzSidebarEntry kz_sidebar_bookmarks =
 	priority_hint: 0,
 	label:         N_("Bookmarks"),
 	icon:          NULL,
-	create:        kz_bookmarks_sidebar_new,
+	create:        kz_bookmarks_sidebar_new_with_entry,
 };
 
 static GtkVBoxClass *parent_class = NULL;
@@ -196,6 +199,12 @@ kz_bookmarks_sidebar_new (KzSidebar *sidebar)
 	return GTK_WIDGET(bsidebar);
 }
 
+static GtkWidget *
+kz_bookmarks_sidebar_new_with_entry (KzSidebarEntry *entry, KzSidebar *sidebar)
+{
+	return kz_bookmarks_sidebar_new (sidebar);
+}
+
 
 static gboolean
 cb_button_release (GtkWidget *widget, GdkEventButton *event,
